import { Request, Response } from "express";
import GenericService from "../services/GenericService";
import BaseEntity from "../entities/baseEntity";

export default class GenericController<Service extends GenericService<Repository>, Repository extends BaseEntity> {
  constructor(protected readonly service: Service) {}

  async getAll(req: Request, res: Response): Promise<void> {
    let data: Array<Repository> = await this.service.getAll();
    res.json(data);
  }

  async getById(req: Request, res: Response): Promise<void> {
    let id: number = parseInt(req.params.id);
    let data: Repository = await this.service.getById(id);
    res.json(data);
  }

  async create(req: Request, res: Response): Promise<void> {
    let insert = await this.service.insert(req.body);
    res.json(insert.identifiers);
  }

  async update(req: Request, res: Response): Promise<void> {
    let update = await this.service.insert(req.body);
    res.json(update.identifiers);
  }

  async delete(req: Request, res: Response): Promise<void> {
    let id: number = parseInt(req.params.id);
    await this.service.delete(id);
    res.sendStatus(204);
  }
}
